#import "RomController.h"

@implementation RomController

#pragma mark -
#pragma mark In, out, shake it all about

-(id)init
{
	self = [super init];	
	return self;
}


-(void)dealoc
{
	[ROMarray release];
	[recordsFile release];
	[super dealloc];
}


-(void)awakeFromNib
{
	[theTable setDataSource:self];
	[theTable setDelegate:self];
	[theTable setDoubleAction:@selector(doubleClick:)];
	[theTable setTarget:self];
	[theTable setAction:nil];
	[self loadPrefs];
}


#pragma mark -
#pragma mark Delegate functions

// tableView delegates
-(int)numberOfRowsInTableView:(NSTableView *)tableView
{
	//return [ROMarray count];
	int numberOfRows = 0;
	
	if ( [ROMarray count] > 17 )
	{
		numberOfRows = 17;
	}
	return numberOfRows;
}

-(id)tableView:(NSTableView *)tableView objectValueForTableColumn:(NSTableColumn *)tableColumn row:(int)row
{
	[theTable reloadData];
	return [ROMarray objectAtIndex:row+start];
}
// -------------------

-(BOOL)applicationShouldTerminateAfterLastWindowClosed:(NSApplication *)theApplication
{
	return YES;
}


#pragma mark -
#pragma mark Load and Save prefs

- (void)loadPrefs
{
	recordsFile = [NSString stringWithString:@"~/Library/Preferences/ROMmanager.plist"];
	recordsFile = [recordsFile stringByExpandingTildeInPath];
	[recordsFile retain];
	records = [[NSMutableArray alloc] initWithContentsOfFile:recordsFile];
	
	if (nil == records)
	{
		records = [[NSMutableArray alloc] init];
	}
	
	// add a record to the array if needs be
	if ([records count] == nil)
	{
		[records insertObject:@"" atIndex:0];
	}

	// check to see if record 0 is occupied and load from it 
	if ([records objectAtIndex:0] != nil)
	{
		[ROMarray release];
		ROMarray = [[NSMutableArray alloc] init];
		
		[self setPathToConfigFile:[records objectAtIndex:0]];
		[self setFileString: [NSString stringWithContentsOfFile:[self pathToConfigFile]]];

		NSArray *tempArray = [[self fileString] componentsSeparatedByString:@"\r\n"];		
		
		[ROMarray addObjectsFromArray:tempArray];
		[theTable reloadData];
		
		if( [ROMarray count] < 68 )
		{
			[self setPathToConfigFile:@""];
		}
		
		// update the information string at the bottom
		[infoString setStringValue:pathToConfigFile ];
	}
	[self savePrefs];
}


- (void)savePrefs
{
	if ([[self recordsFile] length] != nil && [self pathToConfigFile] != @"")
	{
		NSLog(@"Writing to %@", [self recordsFile]);
		[records writeToFile:recordsFile atomically:YES];
	}
}


#pragma mark -
#pragma mark Accessor methods

-(NSString *)fileString
{
	return fileString;
}

-(void)setFileString:(NSString *)aString
{
	[aString retain];
	[fileString release];
	fileString = aString;
}

-(NSString *)pathToConfigFile
{
	return pathToConfigFile;
}

-(void)setPathToConfigFile:(NSString *)aString
{
	[aString retain];
	[pathToConfigFile release];
	pathToConfigFile = aString;
	[infoString setStringValue:pathToConfigFile ];
}

-(NSString *)recordsFile
{
	return recordsFile;
}

-(void)setRecordsFile:(NSString *)aString
{
	[aString retain];
	[recordsFile release];
	recordsFile = aString;
}
// end of Accessor methods


#pragma mark -
#pragma mark Model selection

- (IBAction)changeSelectedModel:(id)sender
{
	id theCell = [sender selectedCell];
	int theTag = [theCell tag];
	start = 17 * theTag;
	[theTable reloadData];	
}


#pragma mark -
#pragma mark ROM Bank Selectors

-(IBAction)makeEmpty:(id)sender
{
	id theCell = [sender selectedCell];
	int theTag = [theCell tag];
	[ROMarray replaceObjectAtIndex:theTag+start withObject:@"EMPTY"];
	[theTable reloadData];
}

-(IBAction)makeRAM:(id)sender
{
	id theCell = [sender selectedCell];
	int theTag = [theCell tag];
	[ROMarray replaceObjectAtIndex:theTag+start withObject:@"RAM"];
	[theTable reloadData];
}

-(void)makeROM:(int)row
{	
	//NSLog(@"sender  = %d", [sender clickedRow]);
	NSLog(@"length = %d", [fileString length]);
	
	int result;
	
	//------

	if ( [fileString length] != nil )
	{
		NSArray *fileTypes = [NSArray arrayWithObject:@"ROM"];
		NSOpenPanel *oPanel = [NSOpenPanel openPanel];
		
		
		[oPanel setAllowsMultipleSelection:NO];

		
		result = [oPanel runModalForDirectory:pathToConfigFile file:nil  types:fileTypes];
		
		if (result == NSOKButton)
		{
			NSArray *filesToOpen = [oPanel filenames];
			int i, count = [filesToOpen count];
			
			for (i=0; i<count; i++)
			{
				NSString *aFile = [filesToOpen objectAtIndex:i];
				NSLog(@"%@", aFile);
				
				// find the bit we're interested in
				
				NSRange range = [aFile rangeOfString:@"BeebFile" options:NSCaseInsensitiveSearch];

				if ( range.location != NSNotFound )
				{
					NSLog(@"Starts at position %d", range.location );
				}
				else
				{
					NSLog(@"Not Found");
				}
				
				NSLog(@"aFile = %@", aFile);
				aFile = [self changePath:aFile];
								NSLog(@"aFile = %@", aFile);
				
				if (range.location != NSNotFound ) // Lives within the BeebFile folder
				{
					NSString *theBitWeWant = [aFile substringFromIndex:range.location+9];
					[ROMarray replaceObjectAtIndex:(row+start) withObject:theBitWeWant];
				}
				else // Doesn't live in the BeebFile folder
				{
					[ROMarray replaceObjectAtIndex:(row+start) withObject:aFile];
				}
				
				[theTable selectRow:row byExtendingSelection:NO];
				//[theTable selectRow:nil];
				[theTable reloadData];
				
			}		
		}
	}
	
	//------
	
}

#pragma mark -
#pragma mark Load and Save Config File
-(IBAction)loadConfigFile:(id)sender
{
	int result;

	NSArray *fileTypes = [NSArray arrayWithObject:@"cfg"];
	NSOpenPanel *oPanel = [NSOpenPanel openPanel];

	
	[oPanel setAllowsMultipleSelection:NO];
	[oPanel setCanChooseFiles:NO];
	[oPanel setCanChooseDirectories:YES];
	[oPanel setTitle:@"Please select your BeebEm folder"];
	
	result = [oPanel runModalForDirectory:NSHomeDirectory() file:nil  types:fileTypes];
	
	if (result == NSOKButton)
	{
		NSArray *filesToOpen = [oPanel filenames];
		int i, count = [filesToOpen count];
		
		for (i=0; i<count; i++)
		{
			NSString *aFile = [filesToOpen objectAtIndex:i];
			aFile = [aFile stringByAppendingPathComponent:@"/Roms.cfg"];
			NSLog(@"aFile = %@", aFile);
			
			[self setPathToConfigFile: aFile];

			[self setFileString: [NSString stringWithContentsOfFile:aFile]];

					
			NSArray *tempArray = [[self fileString] componentsSeparatedByString:@"\r\n"];
			
			[ROMarray release];
			ROMarray = [[NSMutableArray alloc] init];
			
			[ROMarray addObjectsFromArray:tempArray];
			
			[theTable reloadData];
			NSLog(@"%d entries", [ROMarray count]);
			if( [ROMarray count] < 68 )
			{
				aFile = nil;
			}
			 
			// now save the prefs
			[records replaceObjectAtIndex:0  withObject:aFile];
			NSLog(@"Saving prefs to %@", [records objectAtIndex:0]);
			[self savePrefs];
		}
		
	
	}
}

- (IBAction)saveConfigFile:(id)sender
{
	if ([[self pathToConfigFile] length] != nil )
	{
		NSLog(@"Saving to %@", [self pathToConfigFile]);
		NSString *string = [ROMarray componentsJoinedByString:@"\r\n"];
		[string writeToFile:pathToConfigFile atomically:YES];
	}
}

#pragma mark -
#pragma mark Misc. stuff.

-(IBAction)doubleClick:(id)sender
{
	int row = [sender clickedRow ];
	[self makeROM:row];
	// nothing here
}

-(IBAction)ROMclick:(id)sender
{
	id theCell = [sender selectedCell];
	int row = [theCell tag];
	NSLog(@"theTag is %d", row);
	[self makeROM:row];
}

#pragma mark -
-(NSString *)changePath:(NSString *)aString
{
	NSMutableString *newString = [[NSMutableString alloc] initWithString:aString];
	[newString autorelease];

	NSRange range = [newString rangeOfString:@"/" options:NSCaseInsensitiveSearch];

	
	while (range.location != NSNotFound)
	{
		[newString replaceCharactersInRange:range withString:@"\\"];
		range = [newString rangeOfString:@"/" options:NSCaseInsensitiveSearch];
	}

	return newString;
}

@end
